// Sets up all fields that require inline chooser support.
jQuery(function() {
	jQuery("input[_chooserConfig]").each(_configureInlineChooser);
});

function _configureInlineChooser() {
	// Configure jsonSuggest
	var jsonSuggestSettings = {
		"minCharacters":  1,
		"wildCard":       "%",
		"ajaxResults":    true,
		"onSelect":       itemSelected,
		"onError":        handleError,
		"keyDelay":       500,
		"customSettings": getCustomSettings,
		"width":          "auto",
		"maxHeight":      100
	};	
	var input=this;	
	jQuery(this).jsonSuggest(search, jsonSuggestSettings);

	// Custom settings
	function getCustomSettings(field) {
		// Note: unset options will contain an empty string
		var config = String(field.getAttribute("_chooserConfig")).split(",");
		return {
			"chooserOID":        config[0],
			"targetType":        config[1],
			"actionTypeID":      config[2],
			"bindingFilter":     config[3],
			"boundAttrForValue": config[4],
			"formIdx":           parseInt(config[5]),
			"valueFieldName":    config[6],
			"resetOnSelect":     (config[7] == "true"),
			"savedSearchOID":    config[8],
			"ssceOID":           config[9],
			"aldOID":            config[10],
			"rootEntityOID":     config[11],
			"postback":          (config[12] == "true"),
			"enableQuickCreate": config[13]
		};
	}
	var createLinkId = "";
	var createLinkObj = {};
	var eType = "";
	var ajaxOptions = {};
	var enableQuickCreate = false;

	// Data retriever
	function search(field, searchTerm, wildcard, caseSensitive, notChar, customSettings, searchComplete) {
		if (search.inProgess) {
			search.xhr.abort();
		}
		
		ajaxOptions = {
			"async":    true,
			"cache":    false,
			"data":     jQuery.extend({ "searchTerm": searchTerm }, customSettings),
			"dataType": "text",
			"error":    searchError,
			"success":  searchSuccess,
			"type":     "POST",
			"url":      _inlineRequestHandlerURL,
			"xhr":      getXhr
		};
		jQuery.ajax(ajaxOptions);
		search.inProgress = true;
		
		// _chooserConfig holds a comma-delimited string, the second one is the type name
		eType = String(field.getAttribute("_chooserConfig")).split(",")[1];
		var enableQuickCreateConfig = String(field.getAttribute("_chooserConfig")).split(",")[13];
		if(enableQuickCreateConfig != undefined && enableQuickCreateConfig != null){
			enableQuickCreate = enableQuickCreateConfig.toLowerCase();
		}
		
		function searchSuccess(data, status, xhr) {			
			search.inProgress = false;
			var showCreatePersonLink = $('#showCreatePersonLink').is(':checked');
			var showCreateCompanyLink = $('#showCreateCompanyLink').is(':checked');
			if (xhr.status == 200) {
				try {			
				    if(enableQuickCreate == "true" && ((eType == "Person" && showCreatePersonLink) || (eType == "Company" && showCreateCompanyLink)) && data==="[]") {
				    	$(".jsonSuggestResults").hide();
				    	showAndProcessQuickCreate();
					}else{							
					   searchComplete(data);
				    }
					
				} catch (e) {
					handleError(field, "An unexpected error has occurred.<br>Please try the pop-up chooser. " + e.description);
				}
			}
		}
		
		function searchError(xhr, status, error) {
			search.inProgress = false;
			searchComplete("[]");
			if (status != "timeout") {
				handleError(field, "An unexpected error has occurred.<br>Please try the pop-up chooser.");
			}
		}
		
		function showAndProcessQuickCreate() {
			//Provide a link to create a contact or organization in the json results table
			
			//Unique Id for the quick create link that we add below
			createLinkId = $(input).data("jsonSuggestCustomSettings").valueFieldName + "_quickLink";
		    
			var anchorLinkText= "<a id=" + createLinkId + " class=\"quickCreate\" style=\"cursor : pointer; font-style: italic; color: blue; text-decoration: underline\" tabindex=0>Create new";
			
			//Declare an object to hold different variables and values based on contact or organization
			var hostName = "//" + location.hostname + "/" + $(location).prop('pathname').split('/')[1] + "";      				
			if(eType === "Person"){
				anchorLinkText += " contact</a>";
				createLinkObj.headers	  = ["Last", "First", "Organization"];
				createLinkObj.bindings 	  = ["lastName", "firstName", "employer.name"];
				createLinkObj.data 		  = ["No such user.", "", anchorLinkText];
				createLinkObj.url		  = hostName + "/RMConsole/Person/PersonQuickForm?_webrNew=all&SuppressPortalTools=1";
				createLinkObj.dialogTitle = "New Contact";
			} else {
				anchorLinkText += " organization</a>";
				createLinkObj.headers 	  = ["Organization", "Org Parent"];
				createLinkObj.bindings 	  = ["name", "parent"];
				createLinkObj.data 		  = ["No such organization.", anchorLinkText];
				createLinkObj.url		  = hostName + "/RMConsole/Organization/CompanyQuickForm?_webrNew=all&SuppressPortalTools=1";
				createLinkObj.dialogTitle = "New Organization";
			}
			
			//Construct a JSON object, send it to jsonSuggest so that it builds the table
			var showCreateLinkObj = [
				{
					"headers"	: createLinkObj.headers,
					"bindings"	: createLinkObj.bindings,
					"filters" : [0, 1],
					"sortCol" : 0,
					"sortAsc" : true
				},
				{
					"data" : createLinkObj.data,
					"displayString" : "",
					"oid" : ""
				}
			];
			searchComplete(JSON.stringify(showCreateLinkObj));
			
			//Attach the event handler to the quick create link
			var createLinkElement = $("#" + PortalTools.escapeJQuerySelector(createLinkId));
			createLinkElement.on("click keypress", function(e){
				createLinkHandler(e, createLinkElement);
			});
		}
  		
		function getXhr() {
			if (!search.xhr) {
				search.xhr = jQuery.ajaxSettings.xhr();
			}
			return search.xhr;
		}
	}
	
	//Event handler for the create link click and keypress events
	function createLinkHandler(event, element)
	{
		event.preventDefault();
		event.stopPropagation();
		var attrName = $(input).data("jsonSuggestCustomSettings").valueFieldName;
		var temp = _webrUnencodedUrl;
		var dia = $("<div id=\"" + createLinkId + "_dialog\">Please wait, loading...<img src="+PortalTools.busyGifPath+"></img></div>");
		dia.dialog({
			modal: false,
			height: "auto",
			width: "auto",
			title: createLinkObj.dialogTitle,
			position: {
				my: "left top+5",
				of: element
			}, 
			overlay: true,
			buttons: [
	            {
	                text: "Ok",
	                id: attrName + "_OK",
	                click: function() {
	                	if(eType === "Person"){
	                		createLinkObj.quickForm 	= $("#frmPerson");
	                		createLinkObj.errorElement 	= $("#createPersonError");
	                		createLinkObj.name 			= $("#frmPerson [name = 'Person.lastName']");
	                	} else {
	                		createLinkObj.quickForm 	= $("#quickFrmCompany");
	                		createLinkObj.errorElement 	= $("#createOrgError");
	                		createLinkObj.name 			= ($("#quickFrmCompany [name = 'Company.name']").length != 0) 
	                										? $("#quickFrmCompany [name = 'Company.name']") 
	                										: $("#quickFrmCompany [name = 'Company.ID']");
	                	}
	                	$.ajax({
							type: "POST",
							url: _webrUnencodedUrl,
							data: createLinkObj.quickForm.serialize(),
							success: function(data, status, xhr) {
								dia.dialog("close");
								$(input).val(createLinkObj.name.val());
								ajaxOptions["data"] = jQuery.extend({ "searchTerm" : createLinkObj.name.val() }, $(input).data("jsonSuggestCustomSettings")); 
					        	jQuery.ajax(ajaxOptions);
					        },
							error: function(e) {        
								createLinkObj.errorElement.html("Failed to create " + eType + ": " + e.statusText);
								createLinkObj.errorElement.show();
							}
						});
					}
	            },
	            {
	                text: "Cancel",
	                id: attrName + "_Cancel",
	                click: function() {
	                    dia.dialog("close");
	                }
	            }
	        ],
	        open: function(event, ui){
	        	//Focus on the first input field on this dialog
	        	setTimeout(function() {
	       			$("#" + PortalTools.escapeJQuerySelector(createLinkId) + "_dialog :input:visible:first").focus();
	       		}, 800);
	       	},
			/*
			Jquery has a bug in version 1.10.x where the UI dialog steals the focus upon close and throws an error - http://bugs.jqueryui.com/ticket/9101
			The bug was fixed in the version 1.11.0. The below statement for beforeClose can be removed when we upgrade to 1.11.0
			*/ 
			beforeClose: function(event,ui){
				$(document).unbind("focusin.dialog");
			},   					                       			  
			close: function(event,ui){
				$(this).dialog('destroy').remove();
				 _webrUnencodedUrl=temp;
				 $(input).focus();
			} 
		}); 
	    //Add z-index to the dialog so that it is accessible in case of inline cdt views
		if ($(event.target).closest(".InlineCdtViewContainer").length != 0) {
		    $(".ui-dialog").css({
		        "cursor": "default",
		        "z-index": "99"
		    });
		}
	    dia.load(createLinkObj.url, function(){
		    dia.next(".ui-dialog-buttonpane").append(function(){
		    	return "<SPAN class=\"Required\">* </SPAN>";
		    });
		    dia.next(".ui-dialog-buttonpane").append("Required");
		});
	}

	// Selection handler
	function itemSelected(field, item, customSettings) {
		if(item.data[0].indexOf("No such") != -1){
			createLinkHandler(jQuery.Event("onselect"), field);
			return; //No data to update the control
		}
		if (customSettings.resetOnSelect) field.value = "";
		chooserCallback(
			customSettings.postback,
			customSettings.formIdx,
			[item.oid],
			customSettings.valueFieldName,
			[item.displayString],
			null,
			false,
			customSettings.aldOID,
			customSettings.postback ?
				function(chooser) {
					if (customSettings.resetOnSelect) {
						chooser.focus();
						chooser.select(); // To ensure that the input cursor is visible in IE.
					} else {
						jQuery("#" + customSettings.aldOID.replace(/(\.|\[|\])/g, "\\$1") + "_selectBtn").focus();
					}
				} :
				null);
	}
	// Error callback
	function handleError(field, errorMessage) {
		jQuery(field).hide().after("<span class='Error'>" + errorMessage + "</span>");
	}
}