/**
 * Launches the EntityChooser.
 *
 * For documentation, see Common Administration:Choosers/Entity/LauncherSymlink
 */
function launchChooser(sTargetType, sActionTypeID, sSavedSearchOID, sSavedSearchContextEntityOID, sContextFilter,
                       sSelectionMode, bPostback, iFormIdx, sValueFieldName, sDisplayFieldName, sBoundAttributeForValue,
                       sSelectAllHandler, sBindingFilter, sALD, sChooserID, sRootEntity)
{
	// Validate arguments.
	sSelectionMode = String(sSelectionMode).toLowerCase();
	iFormIdx = parseInt(iFormIdx);
	if (typeof sTargetType != "string" || sTargetType == "") {
		reportError("argument sTargetType is invalid");
		return;
	} else if (typeof sActionTypeID != "string" && sActionTypeID != null) {
		reportError("argument sActionTypeID is invalid");
		return;
	} else if (typeof sSavedSearchOID != "string" && sSavedSearchOID != null) {
		reportError("argument sSavedSearchOID is invalid");
		return;
	} else if (typeof sSavedSearchContextEntityOID != "string" && sSavedSearchContextEntityOID != null) {
		reportError("argument sSavedSearchContextEntityOID is invalid");
		return;
	} else if (typeof sContextFilter != "string" && sContextFilter != null) {
		reportError("argument sContextFilter is invalid");
		return;
	} else if (sSelectionMode != "single" && sSelectionMode != "multiple") {
		reportError("argument sSelectionMode is invalid; legal values are 'single' or 'multiple'");
		return;
	} else if (typeof bPostback != "boolean") {
		reportError("argument bPostback is invalid");
		return;
	} else if (isNaN(iFormIdx) || typeof document.forms[iFormIdx] == "undefined") {
		reportError("argument iFormIdx is invalid");
		return;
	} else if (typeof sValueFieldName != "string"
	        || typeof document.getElementById(sValueFieldName) == "undefined") {	       
		reportError("argument sValueFieldName is invalid");
		return;
	} else if (typeof sDisplayFieldName != "string" && sDisplayFieldName != null) {
		reportError("argument sDisplayFieldName is invalid");
		return;
	// Had to relax the validation here because the view controls are inconsistent in their
	// use of a "display field".  A display field name is frequently passed when the field
	// doesn't exist (e.g., the Person control).
	//} else if (sDisplayFieldName != null && sDisplayFieldName != ""
	//        && typeof document.forms[iFormIdx].elements[sDisplayFieldName] == "undefined") {
	//	reportError("argument sDisplayFieldName is invalid");
	//	return;
	} else if (typeof sBoundAttributeForValue != "undefined" && typeof sBoundAttributeForValue != "string"
	        && sBoundAttributeForValue != null) {
		reportError("argument sBoundAttributeForValue is invalid");
		return;
	} else if (typeof sBindingFilter != "undefined" && typeof sBindingFilter != "string"
	        && sBindingFilter != null) {
		reportError("argument sBindingFilter is invalid");
		return;
	} else if (typeof sALD != "undefined" && typeof sALD != "string"
	        && sALD != null) {
	    reportError("argument sALD is invalid");
	    return;
	} else if (typeof sChooserID != undefined && typeof sChooserID != "string"
	        && sChooserID != null) {
	    reportError("argument sChooserID is invalid");
	    return;
	}else if (typeof sRootEntity != "undefined" && typeof sRootEntity != "string"
	        && sRootEntity != null) {
	    reportError("argument sRootEntity is invalid");
	    return;
	}
	// If the actionTypeID is unspecified, use "Assign".
	if (sActionTypeID == null || sActionTypeID == "") {
		sActionTypeID = "Assign";
	}
	// Ensure that the chooser URL is available.
	if (typeof _g_sChooserUrl == "undefined") {
		reportError("chooser URL is undefined");
		return;
	}
	// Generate the querystring for the chooser.
	var sQueryString = "?targetType=" + escape(sTargetType)
	                 + "&actionTypeID=" + escape(sActionTypeID)
	                 + "&selectionMode=" + sSelectionMode
	                 + "&postback=" + bPostback
	                 + "&formIdx=" + iFormIdx
	                 + "&valueFieldName=" + escape(sValueFieldName);
	var sValueField=document.getElementById(sValueFieldName); 	
	if(sValueField==null) {	
		sValueField=document.getElementsByName(sValueFieldName)[0];	 
	}
	var sValueFieldType = sValueField.type.toUpperCase();
	if (sValueFieldType == "SELECT-ONE" || sValueFieldType == "SELECT-MULTIPLE") {
		// The value field is a select control, so set the display field to the same name.
		sDisplayFieldName = sValueFieldName;
	}
	if (typeof sDisplayFieldName == "string" && sDisplayFieldName != "") {
		sQueryString += "&displayFieldName=" + escape(sDisplayFieldName);
	}
	if (typeof sSavedSearchOID == "string" && sSavedSearchOID != "") {
		sQueryString += "&savedSearchOID=" + escape(sSavedSearchOID);
	} else {
		// The use of a saved search constraint supersedes any initialization performed due
		// to a context filter.
		if (typeof sContextFilter == "string" && sContextFilter != "") {
			sQueryString += "&contextFilter=" + escape(sContextFilter);
		}
	}
	if (typeof sSavedSearchContextEntityOID == "string" && sSavedSearchContextEntityOID != "") {
		sQueryString += "&savedSearchContextEntityOID=" + escape(sSavedSearchContextEntityOID);
	}
	
	if (typeof sBoundAttributeForValue == "string" && sBoundAttributeForValue != "") {
		sQueryString += "&boundAttributeForValue=" + escape(sBoundAttributeForValue);
	}
	if (typeof sBindingFilter == "string" && sBindingFilter != "") {
		sQueryString += "&bindingFilter=" + escape(sBindingFilter);
	}
	if (typeof sALD == "string" && sALD != "") {
		sQueryString += "&ald=" + escape(sALD);
	}
	if (typeof sChooserID == "string" && sChooserID != "") {
		sQueryString += "&chooserID=" + escape(sChooserID);
	}
	if (typeof sRootEntity == "string" && sRootEntity != "") {
		sQueryString += "&rootEntity=" + escape(sRootEntity);
	}
	// Open the chooser window.
	var sChooserUrl = _g_sChooserUrl + sQueryString;
	var sWinName = "_blank";
	var iHeightOffset = 66, iWidthOffset = 10;
	var iWinHeight = Math.min(_g_iWinHeight, screen.availHeight - iHeightOffset);
	var iWinWidth = Math.min(_g_iWinWidth, screen.availWidth - iWidthOffset);
	var arrCtrWinPos = _getCenteredWindowPosition(iWinWidth + iWidthOffset, iWinHeight + iHeightOffset);
	var sWinFeatures = "width=" + iWinWidth + ",height=" + iWinHeight
	                 + ",left=" + arrCtrWinPos[0] + ",top=" + arrCtrWinPos[1]
	                 + ",resizable=yes,scrollbars=yes";	                
	window.open(sChooserUrl, sWinName, sWinFeatures);
	
	function reportError(sErrorMessage)
	{
		alert("EXCEPTION WebrCommon/Scripts/EntityChooser/Launcher.js launchChooser():\n\n"
		    + sErrorMessage);
	}
}

/**
 * Determines the coordinates for centering a window of the specified size.
 * The coordinates are for the upper left corner of the window.  If the window size
 * exceeds the size of the screen, this will return the coordinates [0, 0].
 *
 * This is safe to call with any browser because there is a check to ensure that
 * the requisite features are supported.  This appears to work will all browsers,
 * including Netscape 4.79.
 *
 * @param iWinWidth
 *   Integer - the pixel width of the window.
 * @param iWinHeight
 *   Integer - the pixel height of the window.
 * @return
 *   A two-element array [left, top].
 */ 
function _getCenteredWindowPosition(iWinWidth, iWinHeight)
{
	var iWinLeft = 0;
	var iWinTop  = 0;
	// Ensure that the browser supports the necessary features.
	if (typeof window.screen != "undefined") {
		iWinLeft = (window.screen.availWidth - iWinWidth) / 2;
		if (iWinLeft < 0) iWinLeft = 0;
		iWinTop = (window.screen.availHeight - iWinHeight) / 2;
		if (iWinTop < 0) iWinTop = 0;
   	}
   	return new Array(iWinLeft, iWinTop);
}