/**
 * Note: This relies on Portal "secret-menu" functionality that is not officially supported and is subject to change
 * upon each Portal release. By implementing as a module to be referenced by other scripts, we can ensure that any
 * assumptions are captured here and can be changed as needed.
 * @module ViewReady
 * @type {Deferred<boolean>} A promise that will be resolved when Portal's view setup is actually "ready". This may be
 *  well after the document-ready event. It is resolved with a single boolean argument - whether it used Portal's view
 *  ready (true) or reverted to jQuery's document-ready event due to no Navigation object being present (false).
 */
define(['jquery', 'underscore'], function ($, _) {
	var viewReady = new jQuery.Deferred();
	// use Portal's (secret-menu, not officially supported for custom use as of Portal 8.0) view-ready handler to ensure we
	// don't run anything waiting on "viewReady" to be resolved this until Portal's view-ready handling is complete.
	$(function () {
		// Argh, these modules (or at least Navigation) depend on global variables that won't be set until DOM-ready.
		// So we defer the actual 'requirement' of them until DOM-ready. :(
		require(['EntityView/View', 'EntityView/Navigation'], function (View, Navigation) {
			var nav = Navigation.get();
			var viewId = nav ? nav.viewId : null;
			if (viewId) {
				View.ready(viewId, function () {
					viewReady.resolve(true);
				});
			} else {
				// Revert to jQuery doc-ready as there is no Navigation object available in this context.
				viewReady.resolve(false);
			}
		});
	});
	return viewReady;
});